# -*- coding: utf-8 -*-

import datetime

from django.db import models, IntegrityError
from django.template.defaultfilters import slugify
from django.contrib.auth.models import User

from jmb.core.db.models import DateModel, UserModel


class Metric(DateModel, UserModel):
    """ The type of metric we want to store """
    name = models.CharField(max_length=50)
    slug = models.SlugField(unique=True, max_length=60, db_index=True)

    class Meta:
        verbose_name = 'Metric'
        verbose_name_plural = 'Metrics'
        ordering = ('name',)

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.id and not self.slug:
            self.slug = slug = slugify(self.name)
            i = 0
            while True:
                try:
                    return super(Metric,self).save(*args, **kwargs)
                except IntegrityError:
                    i += 1
                    self.slug = "%s_%d" % (self, i)
        else:
            return super(Metric, self).save(*args, **kwargs)


class MetricSet(DateModel, UserModel):
    """ A set of metrics that should be sent via email to certain users """
    name = models.CharField(max_length=50)
    metrics = models.ManyToManyField(Metric)
    email_recipients = models.ManyToManyField(User)
    no_email = models.BooleanField(default=False)
    send_daily = models.BooleanField(default=True)
    send_weekly = models.BooleanField(default=False)
    send_monthly = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'Metric Set'
        verbose_name_plural = 'Metric Sets'

    def __unicode__(self):
        return self.name


class MetricItem(DateModel, UserModel):
    """ Individual metric items """
    metric = models.ForeignKey(Metric)
    num = models.IntegerField(default=1)
    datetime_created = models.DateTimeField(default=datetime.datetime.now, db_index=True)
    user = models.ForeignKey(User, null=True, blank=True, default=None)

    class Meta:
        verbose_name = 'Metric Item'
        verbose_name_plural = 'Metric Items'
        ordering = ('-id',)

    def __unicode__(self):
        if self.user:
            return "'%s' of %d on %s by %s" % (
                self.metric.name, self.num,
                self.datetime_created, self.user.username
            )
        return "'%s' of %d on %s" % (
            self.metric.name, self.num,
            self.datetime_created
        )


class MetricUserDay(DateModel, UserModel):
    """Aggregation of Metrics on a per day basis, divided by user"""
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)
    user = models.ForeignKey(User)

    class Meta:
        verbose_name = 'Day User Metric'
        verbose_name_plural = 'By Day For User Metrics'
        ordering = ('-created',)

    def __unicode__(self):
        return "%s '%s' for '%s' by '%s'" % (
            self.num, self.metric.name,
            self.created, self.user.username
        )


class MetricDay(DateModel, UserModel):
    """ Aggregation of Metrics on a per day basis """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)

    class Meta:
        verbose_name = 'Day Metric'
        verbose_name_plural = 'By Day Metrics'
        ordering = ('-created',)

    def __unicode__(self):
        return "%s '%s' for '%s'" % (
            self.num, self.metric.name, self.created
        )


class MetricUserWeek(DateModel, UserModel):
    """ Aggregation of Metrics on a weekly basis, divided by user """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)
    user = models.ForeignKey(User)

    class Meta:
        verbose_name = 'Week User Metric'
        verbose_name_plural = 'By Week Metrics For User'
        ordering = ('-created',)

    def get_format_created(self):
        return "for week %s of %s "% (
            self.created.strftime("%U"), self.created.strftime("%Y")
        )

    def __unicode__(self):
        return "%s '%s' %s by %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
            self.user.username
        )


class MetricWeek(DateModel, UserModel):
    """ Aggregation of Metrics on a weekly basis """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)

    class Meta:
        verbose_name = 'Week Metric'
        verbose_name_plural = 'By Week Metrics'
        ordering = ('-created',)

    def get_format_created(self):
        return "for week %s of %s "% (
            self.created.strftime("%U"), self.created.strftime("%Y")
        )

    def __unicode__(self):
        return "%s '%s' %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
    )


class MetricUserMonth(DateModel, UserModel):
    """ Aggregation of Metrics on monthly basis, divided by user """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)
    user = models.ForeignKey(User)

    class Meta:
        verbose_name = 'Month User Metric'
        verbose_name_plural = 'By Month Metrics For User'
        ordering = ('-created',)

    def get_format_created(self):
        return "for %s %s "% (
            self.created.strftime("%B"), self.created.strftime("%Y")
        )

    def __unicode__(self):
        return "%s '%s' %s by %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
            self.user.username,
        )


class MetricMonth(DateModel, UserModel):
    """ Aggregation of Metrics on monthly basis """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)

    class Meta:
        verbose_name = 'Month Metric'
        verbose_name_plural = 'By Month Metrics'
        ordering = ('-created',)

    def get_format_created(self):
        return "for %s %s "% (
            self.created.strftime("%B"), self.created.strftime("%Y")
        )

    def __unicode__(self):
        return "%s '%s' %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
        )


class MetricUserYear(DateModel, UserModel):
    """ Aggregation of Metrics on a yearly basis """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)
    user = models.ForeignKey(User)

    class Meta:
        verbose_name = 'Year User Metric'
        verbose_name_plural = 'By Year Metrics For User'
        ordering = ('-created',)

    def get_format_created(self):
        return "for year %s  "% self.created.strftime("%Y")

    def __unicode__(self):
        return "%s '%s' %s by %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
            self.user.username,
        )


class MetricYear(DateModel, UserModel):
    """ Aggregation of Metrics on a yearly basis """
    metric = models.ForeignKey(Metric)
    num = models.BigIntegerField(default=0)
    created = models.DateField(default=datetime.date.today, db_index=True)

    class Meta:
        verbose_name = 'Year Metric'
        verbose_name_plural = 'By Year Metrics'
        ordering = ('-created',)

    def get_format_created(self):
        return "for year %s  "% self.created.strftime("%Y")

    def __unicode__(self):
        return "%s '%s' %s" % (
            self.num, self.metric.name,
            self.get_format_created(),
        )